//-----------------------------------------------------
// CellShadedTeapotMesh.cs (c) 2007 by Charles Petzold
//-----------------------------------------------------
using System;
using System.Windows.Media.Media3D;

namespace Petzold.Media3D
{
    public class CellShadedTeapotMesh : TeapotMesh
    {
        static Vector3D[] vects = { new Vector3D(1, 0, 0), new Vector3D(-1, 0, 0),
                             //       new Vector3D(0, 1, 0), new Vector3D(0, -1, 0),
                                    new Vector3D(0, 0, 1), new Vector3D(0, 0, -1),

                             //       new Vector3D(1, 1, 0), new Vector3D(-1, 1, 0),
                             //       new Vector3D(1, -1, 0), new Vector3D(-1, -1, 0),

                             //       new Vector3D(1, 0, 1), new Vector3D(1, 0, -1),
                             //       new Vector3D(-1, 0, 1), new Vector3D(-1, 0, -1),

                            //        new Vector3D(0, 1, 1), new Vector3D(0, 1, -1),
                            //        new Vector3D(0, -1, 1), new Vector3D(0, -1, -1)
       
        };

        // Normalize all vectors in vects.
        static CellShadedTeapotMesh()
        {
            for (int i = 0; i < vects.Length; i++)
                vects[i].Normalize();
        }


        // Constructor sets all normals to the closest item in the vects collection.
        public CellShadedTeapotMesh()
        {
            Vector3DCollection normals = Geometry.Normals;

            for (int i = 0; i < normals.Count; i++)
            {
                int indexSmallest = -1;
                double diffSmallest = Double.PositiveInfinity;

                for (int j = 0; j < vects.Length; j++)
                {
                    double diff = DifferenceSquared(normals[i], vects[j]);

                    if (diff < diffSmallest)
                    {
                        indexSmallest = j;
                        diffSmallest = diff;
                    }
                }
                normals[i] = vects[indexSmallest];
            }
        }

        // Method assumes the vectors are normalized!
        double DifferenceSquared(Vector3D v1, Vector3D v2)
        {
            double dx = v1.X - v2.X;
            double dy = v1.Y - v2.Y;
            double dz = v1.Z - v2.Z;

            return dx * dx + dy * dy + dz * dz;
        }

        protected override System.Windows.Freezable CreateInstanceCore()
        {
            return new CellShadedTeapotMesh();
        }
    }
}


